import os
import json
from flask import Flask, render_template, redirect, url_for, request, session, current_app, send_from_directory, Blueprint
from authlib.integrations.flask_client import OAuth
from functools import wraps
import secrets

from process_emails import process_emails

app = Flask(__name__)

# Secret key for session management
app.secret_key = os.urandom(24)

# OAuth Configuration
oauth = OAuth(app)

with open("config.json", "r") as file:
    config_json: dict = json.load(file)

# Keycloak Configuration
keycloak_conf = {
    'server_metadata_url': f"{config_json['issuer_url']}/.well-known/openid-configuration",
    'client_id': config_json["login_client_id"],
    'client_secret': config_json["login_client_secret"],
    'client_kwargs': {
        'scope': 'openid profile email'
    }
}

# Configure Keycloak OAuth
keycloak = oauth.register(
    name='keycloak',
    **keycloak_conf
)

# Login required decorator
def login_required(f):
    @wraps(f)
    def decorated_function(*args, **kwargs):
        if 'user' not in session:
            return redirect(url_for('externals.login'))
        return f(*args, **kwargs)
    return decorated_function

# Create a blueprint for all externals routes
externals = Blueprint('externals', __name__, url_prefix='/externals')

@externals.route('/', methods=['GET', 'POST'])
@login_required
def external_form():
   if request.method == "GET":
       return render_template("post.html", username=session.get('username'), added_email="")
   elif request.method == "POST":
       email = request.form.get("email")
       status_okay, error_string = process_emails(mail_address=email)
       if status_okay:
           return render_template("post.html", username=session.get('username'), added_email=f"<h2>{email} added to the FB1 SSO users!</h2>")
       else:
           return f"<h1>Failure :-(</h1> We couldn't register your email {email}. <br> <h2>Reason:</h2>{error_string} <p><a href=\"https://sso.fb1.uni-bremen.de/externals\">Back to the register form...</a>"

@externals.route('/login')
def login():
    # Generate nonce
    nonce = secrets.token_urlsafe(16)
    session['oauth_nonce'] = nonce

    redirect_uri = f"{config_json['base_url']}/externals/authorize"
    return keycloak.authorize_redirect(
        redirect_uri, 
        nonce=nonce
    )

@externals.route("/static/<path:path>", methods=["GET"])
def serve_static_files(path):
    return send_from_directory("static", path)

@externals.route('/authorize')
def authorize():

    # Retrieve the nonce from the session
    nonce = session.pop('oauth_nonce', None)

    token = keycloak.authorize_access_token()
    
    # Verify the token
    userinfo = keycloak.parse_id_token(token, nonce=nonce)
    
    # Store user information in session
    session['user'] = userinfo
    session['username'] = userinfo.get('preferred_username', 'User')
    
    return redirect(url_for('externals.external_form'))

@externals.route('/logout')
def logout():
    # Clear the session
    session.clear()
    
    # Redirect to Keycloak logout endpoint
    logout_url = (
        f"{keycloak_conf['server_metadata_url'].replace('.well-known/openid-configuration', '')}") + \
        f"protocol/openid-connect/logout?client_id={keycloak_conf['client_id']}&" + \
        f"post_logout_redirect_uri={config_json['base_url']}"
    
    return redirect(logout_url)

# Register the blueprint
app.register_blueprint(externals)

@app.errorhandler(401)
def unauthorized(error):
    return redirect(url_for('externals.login'))

if __name__ == '__main__':
    app.run(host='0.0.0.0', port=80)

